@@ -188,6 +188,41 @@
      },
 #endif /* V4L2_PIX_FMT_YUV565 */
 
+/* bayer formats */
+#ifdef V4L2_PIX_FMT_SRGGB8
+{
+  .name     = "Bayer RGGB 8bit",
+    .fourcc   = V4L2_PIX_FMT_SRGGB8,
+    .depth    = 8,
+    .flags    = 0,
+    },
+#endif /* V4L2_PIX_FMT_SRGGB8 */
+#ifdef V4L2_PIX_FMT_SGRBG8
+{
+  .name     = "Bayer GRBG 8bit",
+    .fourcc   = V4L2_PIX_FMT_SGRBG8,
+    .depth    = 8,
+    .flags    = 0,
+    },
+#endif /* V4L2_PIX_FMT_SGRBG8 */
+#ifdef V4L2_PIX_FMT_SGBRG8
+{
+  .name     = "Bayer GBRG 8bit",
+    .fourcc   = V4L2_PIX_FMT_SGBRG8,
+    .depth    = 8,
+    .flags    = 0,
+    },
+#endif /* V4L2_PIX_FMT_SGBRG8 */
+#ifdef V4L2_PIX_FMT_SBGGR8
+{
+  .name     = "Bayer BA81 8bit",
+    .fourcc   = V4L2_PIX_FMT_SBGGR8,
+    .depth    = 8,
+    .flags    = 0,
+    },
+#endif /* V4L2_PIX_FMT_SBGGR8 */
+
+
 
  /* here come the planar formats */
 {
@@ -238,6 +273,14 @@
      .flags    = FORMAT_FLAGS_PLANAR,
      },
 #endif /* V4L2_PIX_FMT_Y41P */
+#ifdef V4L2_PIX_FMT_NV12
+{
+	.name     = "12 bpp Y/CbCr 4:2:0 ",
+		.fourcc   = V4L2_PIX_FMT_NV12,
+		.depth    = 12,
+		.flags    = FORMAT_FLAGS_PLANAR,
+     },
+#endif /* V4L2_PIX_FMT_NV12 */
 
  /* here come the compressed formats */
 
@@ -361,3 +404,15 @@
      .flags    = FORMAT_FLAGS_COMPRESSED,
      },
 #endif /* V4L2_PIX_FMT_VP8 */
+{
+   .name     = "VP9",
+     .fourcc   = V4L2_PIX_FMT_VP9,
+     .depth    = 32,
+     .flags    = FORMAT_FLAGS_COMPRESSED,
+     },
+{
+   .name     = "HEVC",
+     .fourcc   = V4L2_PIX_FMT_HEVC,
+     .depth    = 32,
+     .flags    = FORMAT_FLAGS_COMPRESSED,
+     },
